package com.lucaszanella.flutterplugincrashrestartkt

import android.app.Activity
import android.app.AlarmManager
import android.app.PendingIntent
import android.content.Context
import android.content.Intent
import android.util.Log
import android.widget.Toast

import java.util.concurrent.TimeUnit


class FlutterExceptionHandler<A>(private val type: Class<A>, private val activity: Activity) : Thread.UncaughtExceptionHandler {
    init {
        Thread.setDefaultUncaughtExceptionHandler(this)
        val intent = activity.intent
        if (intent.getBooleanExtra("crash", false)) {
            val message_ = intent.getStringExtra("message")
            val cause_ = intent.getStringExtra("cause")
            val stackTrace_ = intent.getStringExtra("stackTrace")
            message = if (message_ != null) message_ else ""
            cause = if (cause_ != null) cause_ else ""
            stackTrace = if (stackTrace_ != null) stackTrace_ else ""
            hasStackTrace = true

            Log.d(TAG, "App restarted after crash")
            Log.d(TAG, "message: " + message)
            Log.d(TAG, "cause: " + cause)
            Log.d(TAG, "stackTrace: " + stackTrace)
        }
    }

    override fun uncaughtException(thread: Thread, ex: Throwable) {
        var stackStraceString = ""
        val s = ex.stackTrace
        for (i in s.indices) {
            stackStraceString += s[i].toString() + "\n"
        }
        val intent = Intent(activity, type)
        intent.putExtra("crash", true)
        intent.putExtra("message", ex.message)
        intent.putExtra("cause", ex.toString())
        intent.putExtra("stackTrace", stackStraceString)
        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP
                or Intent.FLAG_ACTIVITY_CLEAR_TASK
                or Intent.FLAG_ACTIVITY_NEW_TASK)
        val pendingIntent = PendingIntent.getActivity(activity.baseContext, 0, intent, PendingIntent.FLAG_ONE_SHOT)
        val mgr = activity.baseContext.getSystemService(Context.ALARM_SERVICE) as AlarmManager
        mgr.set(AlarmManager.RTC, System.currentTimeMillis() + 100, pendingIntent)
        activity.finish()
        System.exit(2)
    }

    companion object {
        private val TAG = "FlutterExceptionHandler"
        private lateinit var thread: Thread
        internal var message: String = ""
        internal var cause: String = ""
        internal var stackTrace: String = ""
        internal var hasStackTrace: Boolean = false

        //Simulates a crash
        fun crashMe() {
            /*
            Crash must happen outside the thread that calls crashMe() because
            Flutter has a default exception handler for method channels:
            https://github.com/flutter/engine/blob/025e2d82dda54af7f33a0d511bde47ec835593b1/shell/platform/android/io/flutter/plugin/common/MethodChannel.java#L224
         */
            thread = Thread(Runnable { throw NullPointerException() })
            thread.start()
        }
    }
}